@@ -48,14 +48,9 @@ class_exprs = { "ObjectID": "h5py.h5.ObjectID",
                 "[Gg]roup creation property list": "h5py.h5p.PropGCID"}
 
 
-try:
-    class_exprs = dict( 
-        (re.compile(class_base % x.replace(" ",r"\s"), re.VERBOSE), y) \
-        for x, y in class_exprs.iteritems() )
-except AttributeError:
-    class_exprs = dict( 
-        (re.compile(class_base % x.replace(" ",r"\s"), re.VERBOSE), y) \
-        for x, y in class_exprs.items() )
+class_exprs = dict(
+    (re.compile(class_base % x.replace(" ",r"\s"), re.VERBOSE), y) \
+    for x, y in class_exprs.items() )
 
 
 def replace_class(istr):
@@ -64,14 +59,9 @@ def replace_class(istr):
         pre, name, post = match.group('pre', 'name', 'post')
         return '%s:class:`%s <%s>`%s' % (pre, name, target, post)
 
-    try:
-        for expr, target in class_exprs.iteritems():
-            rpl2 = partial(rpl, target)
-            istr = safe_replace(istr, expr, rpl2)
-    except AttributeError:
-         for expr, target in class_exprs.items():
-            rpl2 = partial(rpl, target)
-            istr = safe_replace(istr, expr, rpl2)
+    for expr, target in class_exprs.items():
+        rpl2 = partial(rpl, target)
+        istr = safe_replace(istr, expr, rpl2)
 
     return istr
 
@@ -98,7 +88,7 @@ const_expr = re.compile(r"""
 (?P<post>
   \W?                         # May have trailing punctuation
   (?:$|\s+)                   # Must be followed by whitespace or end of string
-)                      
+)
 """ % const_exclude, re.VERBOSE)
 
 def replace_constant(istr, current_module):
@@ -160,17 +150,17 @@ param_expr = re.compile(r"""
 \+
 \s+
 (?P<desc>
-  [^\s\(]
+  [^\s(]
   .*
-  [^\s\)]
+  [^\s)]
 )
 (?:
   \s+
   \(
   (?P<default>
-    [^\s\(]
+    [^\s(]
     .*
-    [^\s\)]
+    [^\s)]
   )
   \)
 )?
@@ -266,4 +256,3 @@ def setup(spx):
 
     spx.connect('autodoc-process-signature', proc_sig)
     spx.connect('autodoc-process-docstring', proc_doc)
-
