@@ -66,8 +66,10 @@ def make_new_dset(parent, shape=None, dtype=None, data=None, name=None,
     # Validate chunk shape
     if isinstance(chunks, int) and not isinstance(chunks, bool):
         chunks = (chunks,)
+    # Logically, the following `zip` could be strict, but it's happening
+    # before we've done checks elsewhere that raise more descriptive errors
     if isinstance(chunks, tuple) and any(
-        chunk > dim for dim, chunk in zip(tmp_shape, chunks) if dim is not None
+        chunk > dim for dim, chunk in zip(tmp_shape, chunks, strict=False) if dim is not None
     ):
         errmsg = "Chunk shape must not be greater than data shape in any dimension. "\
                  "{} is not compatible with {}".format(chunks, shape)
@@ -254,7 +256,7 @@ class AsTypeView(AbstractView):
     """
     def __init__(self, dset, dtype):
         super().__init__(dset)
-        self._dtype = numpy.dtype(dtype)
+        self._dtype = dtype
 
     @property
     def dtype(self):
@@ -367,16 +369,34 @@ class ChunkIterator:
         if source_sel is None:
             # select over entire dataset
             self._sel = tuple(
-                slice(0, self._shape[dim])
-                for dim in range(rank)
+                slice(0, self._shape[dim]) for dim in range(rank)
             )
         else:
-            if isinstance(source_sel, slice):
-                self._sel = (source_sel,)
+            if isinstance(source_sel, (slice, int)):
+                sel = [source_sel]
             else:
-                self._sel = source_sel
-        if len(self._sel) != rank:
-            raise ValueError("Invalid selection - selection region must have same rank as dataset")
+                sel = list(source_sel)
+            if len(sel) != rank:
+                raise ValueError("Invalid selection - selection region must have same rank as dataset")
+            for dim, s in enumerate(sel):
+                start: int | None
+                stop: int | None
+                step: int | None
+                match s:
+                    case int():
+                        start = s
+                        stop = s + 1
+                        step = None
+                    case slice():
+                        start = s.start or 0
+                        stop = s.stop or self._shape[dim]
+                        step = s.step
+                    case _:
+                        # TODO: use typing.assert_never when Python 3.10 is dropped
+                        raise AssertionError(f'{s}: Selection object must be a slice or integer')
+                sel[dim] = slice(start, stop, step)
+                self._sel = tuple(sel)
+
         self._chunk_index = []
         for dim in range(rank):
             s = self._sel[dim]
@@ -437,6 +457,18 @@ class Dataset(HLObject):
 
         >>> double_precision = dataset.astype('f8')[0:100:2]
         """
+        dtype = numpy.dtype(dtype)
+        if dtype == self.dtype:
+            return self
+
+        if dtype.kind == "T":
+            string_info = h5t.check_string_dtype(self.dtype)
+            if string_info is None:
+                raise TypeError(
+                    f"dset.astype({dtype}) can only be used on datasets with "
+                    "an HDF5 string datatype"
+                )
+
         return AsTypeView(self, dtype)
 
     def asstr(self, encoding=None, errors='strict'):
@@ -447,6 +479,12 @@ class Dataset(HLObject):
         The parameters have the same meaning as in ``bytes.decode()``.
         If ``encoding`` is unspecified, it will use the encoding in the HDF5
         datatype (either ascii or utf-8).
+
+        .. note::
+           On NumPy 2.0 and later, it is recommended to use native NumPy
+           variable-width strings instead:
+
+           >>> str_array = dataset.astype('T')[:]
         """
         string_info = h5t.check_string_dtype(self.dtype)
         if string_info is None:
@@ -715,7 +753,7 @@ class Dataset(HLObject):
                 try:
                     newlen = int(size)
                 except TypeError:
-                    raise TypeError("Argument must be a single int if axis is specified")
+                    raise TypeError("Argument must be a single int if axis is specified") from None
                 size = list(self.shape)
                 size[axis] = newlen
 
@@ -1115,39 +1153,33 @@ class Dataset(HLObject):
     @with_phil
     def __repr__(self):
         if not self:
-            r = '<Closed HDF5 dataset>'
+            return "<Closed HDF5 dataset>"
+
+        if self.name is None:
+            name = "(anonymous)"
         else:
-            if self.name is None:
-                namestr = '("anonymous")'
-            else:
-                name = pp.basename(pp.normpath(self.name))
-                namestr = '"%s"' % (name if name != '' else '/')
-            r = '<HDF5 dataset %s: shape %s, type "%s">' % (
-                namestr, self.shape, self.dtype.str
-            )
-        return r
+            name = pp.basename(pp.normpath(self.name))
+            name = f'"{name}"'
 
-    if hasattr(h5d.DatasetID, "refresh"):
-        @with_phil
-        def refresh(self):
-            """ Refresh the dataset metadata by reloading from the file.
+        return f'<HDF5 dataset {name}: shape {self.shape}, type "{self.dtype.str}">'
 
-            This is part of the SWMR features and only exist when the HDF5
-            library version >=1.9.178
-            """
-            self._id.refresh()
-            self._cache_props.clear()
+    @with_phil
+    def refresh(self):
+        """ Refresh the dataset metadata by reloading from the file.
 
-    if hasattr(h5d.DatasetID, "flush"):
-        @with_phil
-        def flush(self):
-            """ Flush the dataset data and metadata to the file.
-            If the dataset is chunked, raw data chunks are written to the file.
+        This is part of the SWMR features.
+        """
+        self._id.refresh()
+        self._cache_props.clear()
 
-            This is part of the SWMR features and only exist when the HDF5
-            library version >=1.9.178
-            """
-            self._id.flush()
+    @with_phil
+    def flush(self):
+        """ Flush the dataset data and metadata to the file.
+        If the dataset is chunked, raw data chunks are written to the file.
+
+        This is part of the SWMR features.
+        """
+        self._id.flush()
 
     if vds_support:
         @property
