@@ -1,21 +1,22 @@
 #!/usr/bin/env python3
 """
-    Implements a custom Distutils build_ext replacement, which handles the
+    Implements a custom build_ext replacement, which handles the
     full extension module build process, from api_gen to C compilation and
     linking.
 """
 
-try:
-    from setuptools import Extension
-except ImportError:
-    from distutils.extension import Extension
-from distutils.command.build_ext import build_ext
 import copy
 import sys
+import sysconfig
 import os
 import os.path as op
+import platform
 from pathlib import Path
 
+from Cython import Tempita as tempita
+from setuptools import Extension
+from setuptools.command.build_ext import build_ext
+
 import api_gen
 from setup_configure import BuildConfig
 
@@ -24,7 +25,6 @@ def localpath(*args):
     return op.abspath(op.join(op.dirname(__file__), *args))
 
 
-MODULES_NUMPY2 = ['_npystrings']
 MODULES = ['defs', '_errors', '_objects', '_proxy', 'h5fd', 'h5z',
             'h5', 'h5i', 'h5r', 'utils', '_selector',
             '_conv', 'h5t', 'h5s',
@@ -32,7 +32,10 @@ MODULES = ['defs', '_errors', '_objects', '_proxy', 'h5fd', 'h5z',
             'h5d', 'h5a', 'h5f', 'h5g',
             'h5l', 'h5o',
             'h5ds', 'h5ac',
-            'h5pl'] + MODULES_NUMPY2
+            'h5pl']
+MODULES_NUMPY2_ONLY = ['_npystrings']
+
+ALL_MODULES = MODULES + MODULES_NUMPY2_ONLY + ["api_types_ext", "api_types_hdf5"]
 
 COMPILER_SETTINGS = {
    'libraries'      : ['hdf5', 'hdf5_hl'],
@@ -73,7 +76,7 @@ if sys.platform.startswith('win'):
 class h5py_build_ext(build_ext):
 
     """
-        Custom distutils command which encapsulates api_gen pre-building,
+        Custom setuptools command which encapsulates api_gen pre-building,
         Cython building, and C compilation.
 
         Also handles making the Extension modules, since we can't rely on
@@ -81,7 +84,7 @@ class h5py_build_ext(build_ext):
     """
 
     @classmethod
-    def _make_extensions(cls, config):
+    def _make_extensions(cls, config, templ_config):
         """ Produce a list of Extension instances which can be passed to
         cythonize().
 
@@ -117,22 +120,37 @@ class h5py_build_ext(build_ext):
         if os.name != 'nt':
             settings['runtime_library_dirs'] = settings['library_dirs']
 
-        return [cls._make_extension(m, settings) for m in MODULES]
+        for module in ALL_MODULES:
+            raw_path = Path(localpath("h5py")).joinpath(module).resolve()
+            for ext in ['.pyx', '.pxd', '.pxi']:
+                if not (templ := raw_path.with_suffix(f'.templ{ext}')).exists():
+                    continue
+
+                if (target := raw_path.with_suffix(ext)).exists():
+                    current_text = target.read_text('utf-8')
+                else:
+                    current_text = ""
+                new_text = tempita.sub(templ.read_text(), **templ_config)
+                if new_text != current_text:
+                    target.write_text(new_text, 'utf-8')
+
+        settings['define_macros'].append(('NPY_TARGET_VERSION', 'NPY_1_21_API_VERSION'))
+        extensions = [cls._make_extension(m, settings) for m in MODULES]
+
+        if int(numpy.__version__.split('.')[0]) >= 2:
+            # Enable NumPy 2.0 C API for modules that require it.
+            # NUMPY2_MODULES will not be importable when NumPy 1.x is installed.
+            settings['define_macros'].append(('NPY_TARGET_VERSION', 'NPY_2_0_API_VERSION'))
+            extensions.extend(cls._make_extension(m, settings) for m in MODULES_NUMPY2_ONLY)
+
+        return extensions
 
     @staticmethod
     def _make_extension(module, settings):
-        import numpy
-
         sources = [localpath('h5py', module + '.pyx')] + EXTRA_SRC.get(module, [])
         settings = copy.deepcopy(settings)
         settings['libraries'] += EXTRA_LIBRARIES.get(module, [])
 
-        assert numpy.__version__ >= '2.0'  # See build dependencies in pyproject.toml
-        if module in MODULES_NUMPY2:
-            # Enable NumPy 2.0 C API for modules that require it.
-            # These modules will not be importable when NumPy 1.x is installed.
-            settings['define_macros'].append(('NPY_TARGET_VERSION', 0x00000012))
-
         return Extension('h5py.' + module, sources, **settings)
 
     def run(self):
@@ -155,58 +173,43 @@ class h5py_build_ext(build_ext):
         config = BuildConfig.from_env()
         config.summarise()
 
-        if config.hdf5_version < (1, 10, 6):
+        if config.hdf5_version < (1, 10, 7) or config.hdf5_version == (1, 12, 0):
             raise Exception(
-                f"This version of h5py requires HDF5 >= 1.10.6 (got version "
+                f"This version of h5py requires HDF5 >= 1.10.7 and != 1.12.0 (got version "
                 f"{config.hdf5_version} from environment variable or library)"
             )
 
-        config_file = localpath('h5py', 'config.pxi')
-
         # Refresh low-level defs if missing or stale
         print("Executing api_gen rebuild of defs")
         api_gen.run()
 
-        # Rewrite config.pxi file if needed
-        s = f"""\
-# This file is automatically generated by the h5py setup script.  Don't modify.
-
-DEF MPI = {bool(config.mpi)}
-DEF ROS3 = {bool(config.ros3)}
-DEF HDF5_VERSION = {config.hdf5_version}
-DEF DIRECT_VFD = {bool(config.direct_vfd)}
-DEF VOL_MIN_HDF5_VERSION = (1,11,5)
-DEF COMPLEX256_SUPPORT = {complex256_support}
-DEF NUMPY_BUILD_VERSION = '{numpy.__version__}'
-DEF CYTHON_BUILD_VERSION = '{cython_version}'
-"""
-        write_if_changed(config_file, s)
-
+        templ_config = {
+            "MPI": bool(config.mpi),
+            "ROS3": bool(config.ros3),
+            "HDF5_VERSION": config.hdf5_version,
+            "DIRECT_VFD": bool(config.direct_vfd),
+            "VOL_MIN_HDF5_VERSION": (1, 11, 5),
+            "COMPLEX256_SUPPORT": complex256_support,
+            "NUMPY_BUILD_VERSION": numpy.__version__,
+            "NUMPY_BUILD_VERSION_TUPLE": tuple(int(x) for x in numpy.__version__.split('.')[:3]),
+            "CYTHON_BUILD_VERSION": cython_version,
+            "PLATFORM_SYSTEM": platform.system(),
+            "OBJECTS_USE_LOCKING": True,
+            "OBJECTS_DEBUG_ID": False,
+            "FREE_THREADING": sysconfig.get_config_var("Py_GIL_DISABLED") == 1,
+        }
         # Run Cython
         print("Executing cythonize()")
-        self.extensions = cythonize(self._make_extensions(config),
-                                    force=config.changed() or self.force,
-                                    language_level=3)
+        self.extensions = self.distribution.ext_modules = cythonize(
+            self._make_extensions(config, templ_config),
+            force=config.changed() or self.force,
+            language_level=3
+        )
 
         # Perform the build
-        build_ext.run(self)
+        self.swig_opts = None # workaround https://github.com/pypa/setuptools/pull/5083
+        self.finalize_options()
+        super().run()
 
         # Record the configuration we built
         config.record_built()
-
-
-def write_if_changed(target_path, s: str):
-    """Overwrite target_path unless the contents already match s
-
-    Avoids changing the mtime when we're just writing the same data.
-    """
-    p = Path(target_path)
-    b = s.encode('utf-8')
-    try:
-        if p.read_bytes() == b:
-            return
-    except FileNotFoundError:
-        pass
-
-    p.write_bytes(b)
-    print(f'Updated {p}')
